@@ -2,7 +2,7 @@
 //
 // pgAdmin III - PostgreSQL Tools
 //
-// Copyright (C) 2002 - 2012, The pgAdmin Development Team
+// Copyright (C) 2002 - 2013, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 // ctlSQLGrid.cpp - SQL Query result window
@@ -25,7 +25,6 @@
 #define EXTRAEXTENT_WIDTH  6
 
 BEGIN_EVENT_TABLE(ctlSQLGrid, wxGrid)
-	EVT_MENU(MNU_COPY, ctlSQLGrid::OnCopy)
 	EVT_MOUSEWHEEL(ctlSQLGrid::OnMouseWheel)
 END_EVENT_TABLE()
 
@@ -51,11 +50,6 @@ ctlSQLGrid::ctlSQLGrid(wxWindow *parent, wxWindowID id, const wxPoint &pos, cons
 	SetColLabelSize(fntLabel.GetPointSize() * 4);
 	SetDefaultCellOverflow(false);
 
-	wxAcceleratorEntry entries[1];
-	entries[0].Set(wxACCEL_CTRL,                (int)'C',      MNU_COPY);
-	wxAcceleratorTable accel(1, entries);
-	SetAcceleratorTable(accel);
-
 	Connect(wxID_ANY, wxEVT_GRID_LABEL_LEFT_DCLICK, wxGridEventHandler(ctlSQLGrid::OnLabelDoubleClick));
 }
 
@@ -149,14 +143,63 @@ wxString ctlSQLGrid::GetExportLine(int row, wxArrayInt cols)
 	return str;
 }
 
+wxString ctlSQLGrid::GetColumnName(int colNum)
+{
+	wxString columnName = GetColLabelValue(colNum);
+	columnName = columnName.Left(columnName.find(wxT("\n")));
+	return columnName;
+}
+
+void ctlSQLGrid::AppendColumnHeader(wxString &str, int start, int end)
+{
+	size_t i, arrsize;
+	arrsize = (end - start + 1);
+	wxArrayInt columns;
+
+	for(i = 0; i < arrsize; i++)
+	{
+		columns.Add(start + i);
+	}
+
+	AppendColumnHeader(str, columns);
+}
+
+void ctlSQLGrid::AppendColumnHeader(wxString &str, wxArrayInt columns)
+{
+	if(settings->GetColumnNames())
+	{
+		bool CopyQuoting = (settings->GetCopyQuoting() == 1 || settings->GetCopyQuoting() == 2);
+		size_t i;
+
+		for(i = 0; i < columns.Count() ; i++)
+		{
+			long columnPos = columns.Item(i);
+			if(i > 0)
+				str.Append(settings->GetCopyColSeparator());
+
+			if(CopyQuoting)
+				str.Append(settings->GetCopyQuoteChar());
+			str.Append(GetColumnName(columnPos));
+			if(CopyQuoting)
+				str.Append(settings->GetCopyQuoteChar());
+
+		}
+		str.Append(END_OF_LINE);
+	}
+}
+
 int ctlSQLGrid::Copy()
 {
 	wxString str;
 	int copied = 0;
 	size_t i;
 
+
+
 	if (GetSelectedRows().GetCount())
 	{
+		AppendColumnHeader(str, 0, (GetNumberCols() - 1));
+
 		wxArrayInt rows = GetSelectedRows();
 
 		for (i = 0 ; i < rows.GetCount() ; i++)
@@ -174,6 +217,8 @@ int ctlSQLGrid::Copy()
 		wxArrayInt cols = GetSelectedCols();
 		size_t numRows = GetNumberRows();
 
+		AppendColumnHeader(str, cols);
+
 		for (i = 0 ; i < numRows ; i++)
 		{
 			str.Append(GetExportLine(i, cols));
@@ -194,6 +239,8 @@ int ctlSQLGrid::Copy()
 		y1 = GetSelectionBlockTopLeft()[0].GetRow();
 		y2 = GetSelectionBlockBottomRight()[0].GetRow();
 
+		AppendColumnHeader(str, x1, x2);
+
 		for (i = y1; i <= y2; i++)
 		{
 			str.Append(GetExportLine(i, x1, x2));
@@ -211,6 +258,8 @@ int ctlSQLGrid::Copy()
 		row = GetGridCursorRow();
 		col = GetGridCursorCol();
 
+		AppendColumnHeader(str, col, col);
+
 		str.Append(GetExportLine(row, col, col));
 		copied = 1;
 	}
