@@ -2,7 +2,7 @@
 //
 // pgAdmin III - PostgreSQL Tools
 //
-// Copyright (C) 2002 - 2012, The pgAdmin Development Team
+// Copyright (C) 2002 - 2013, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 // ctlSQLResult.cpp - SQL Query result window
@@ -27,7 +27,7 @@ ctlSQLResult::ctlSQLResult(wxWindow *parent, pgConn *_conn, wxWindowID id, const
 	: ctlSQLGrid(parent, id, pos, size)
 {
 	conn = _conn;
-	thread = 0;
+	thread = NULL;
 
 	SetTable(new sqlResultTable(), true);
 
@@ -42,6 +42,18 @@ ctlSQLResult::ctlSQLResult(wxWindow *parent, pgConn *_conn, wxWindowID id, const
 ctlSQLResult::~ctlSQLResult()
 {
 	Abort();
+
+	if (thread)
+	{
+		if (thread->IsRunning())
+		{
+			thread->CancelExecution();
+			thread->Wait();
+		}
+
+		delete thread;
+		thread = NULL;
+	}
 }
 
 
@@ -141,10 +153,15 @@ int ctlSQLResult::Abort()
 	{
 		((sqlResultTable *)GetTable())->SetThread(0);
 
-		thread->Delete();
+		if (thread->IsRunning())
+		{
+			thread->CancelExecution();
+			thread->Wait();
+		}
+
 		delete thread;
+		thread = NULL;
 	}
-	thread = 0;
 	return 0;
 }
 
@@ -219,6 +236,15 @@ void ctlSQLResult::DisplayData(bool single)
 				w = -1;
 
 			SetColSize(col, w);
+			if (thread->DataSet()->ColTypClass(col) == PGTYPCLASS_NUMERIC)
+			{
+				/*
+				 * For numeric columns, set alignment to right.
+				 */
+				wxGridCellAttr *attr = new wxGridCellAttr();
+				attr->SetAlignment(wxALIGN_RIGHT, wxALIGN_TOP);
+				SetColAttr(col, attr);
+			}
 		}
 	}
 	Thaw();
@@ -241,7 +267,12 @@ wxString ctlSQLResult::GetErrorMessage()
 
 pgError ctlSQLResult::GetResultError()
 {
-	return conn->GetLastResultError();
+	if (thread)
+		return thread->GetResultError();
+
+	pgError dummy;
+	memset(&dummy, 0, sizeof(dummy));
+	return dummy;
 }
 
 long ctlSQLResult::NumRows() const
@@ -317,7 +348,44 @@ wxString sqlResultTable::GetValue(int row, int col)
 			if (settings->GetIndicateNull() && thread->DataSet()->IsNull(col))
 				return wxT("<NULL>");
 			else
-				return thread->DataSet()->GetVal(col);
+			{
+
+				wxString decimalMark = wxT(".");
+				wxString s = wxEmptyString;
+				s = thread->DataSet()->GetVal(col);
+
+				if(thread->DataSet()->ColTypClass(col) == PGTYPCLASS_NUMERIC &&
+				        settings->GetDecimalMark().Length() > 0)
+				{
+					decimalMark = settings->GetDecimalMark();
+					s.Replace(wxT("."), decimalMark);
+
+				}
+				if (thread->DataSet()->ColTypClass(col) == PGTYPCLASS_NUMERIC &&
+				        settings->GetThousandsSeparator().Length() > 0)
+				{
+					/* Add thousands separator */
+					size_t pos = s.find(decimalMark);
+					if (pos == wxString::npos)
+						pos = s.length();
+					while (pos > 3)
+					{
+						pos -= 3;
+						if (pos > 1 || !s.StartsWith(wxT("-")))
+							s.insert(pos, settings->GetThousandsSeparator());
+					}
+					return s;
+				}
+				else
+				{
+					wxString data = thread->DataSet()->GetVal(col);
+
+					if (data.Length() > settings->GetMaxColSize())
+						return thread->DataSet()->GetVal(col).Left(settings->GetMaxColSize()) + wxT(" (...)");
+					else
+						return thread->DataSet()->GetVal(col);
+				}
+			}
 		}
 		else
 			return thread->DataSet()->ColName(col);
@@ -327,7 +395,7 @@ wxString sqlResultTable::GetValue(int row, int col)
 
 sqlResultTable::sqlResultTable()
 {
-	thread = 0;
+	thread = NULL;
 }
 
 int sqlResultTable::GetNumberRows()
